package com.dragoninfo.dcuc.app.enumresources;

import java.util.ArrayList;
import java.util.List;

/**
 * 数据资源类型枚举类
 *
 * @author mazq
 * @date 2021/4/13
 */
public enum DataResourceEnum {

    /**
     * 数据分级
     */
    DATA_LEVEL("DATA_LEVEL", "数据分级", "DATA_LEVEL", DataAttrTypeEnum.DATA_ATTR_TABLE),

    /**
     * 数据资源分类
     */
    DATA_RESOURCE_CLASSIFY("DATA_RESOURCE_CLASSIFY", "数据资源分类", "DATA_RESOURCE_CLASSIFY", DataAttrTypeEnum.DATA_ATTR_TABLE),

    /**
     * 数据安全级别
     */
    COLUMN_SECURITY_LEVEL("COLUMN_SECURITY_LEVEL", "数据安全级别", "COLUMN_SECURITY_LEVEL", DataAttrTypeEnum.DATA_ATTR_COLUMN),

    /**
     * 字段分类
     */
    COLUMN_CLASSIFY("COLUMN_CLASSIFY", "字段分类", "COLUMN_CLASSIFY", DataAttrTypeEnum.DATA_ATTR_COLUMN),

    /**
     * 字段关系
     */
    COLUMN_RELATION_CATALOG("COLUMN_RELATION_CATALOG", "字段关系", "COLUMN_RELATION_CATALOG", DataAttrTypeEnum.DATA_ATTR_COLUMN);

    DataResourceEnum(String code, String label, String dataType, DataAttrTypeEnum attrType) {
        this.code = code;
        this.label = label;
        this.dataType = dataType;
        this.attrType = attrType;
    }

    /**
     * 数据资源code
     */
    private String code;

    /**
     * 数据资源名称
     */
    private String label;

    /**
     * 数据资源类型
     * 数据安全级别：DATA_SECURITY_LEVEL
     * 数据资源分类：DATA_RESOURCE_CLASSIFY
     * 数据分级：DATA_CLASSIFY
     * 字段分类：COLUMN_CLASSIFY
     */
    private String dataType;

    /**
     * 数据属性类型
     */
    private DataAttrTypeEnum attrType;

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public String getDataType() {
        return dataType;
    }

    public void setDataType(String dataType) {
        this.dataType = dataType;
    }

    public DataAttrTypeEnum getAttrType() {
        return attrType;
    }

    public void setAttrType(DataAttrTypeEnum attrType) {
        this.attrType = attrType;
    }

    /**
     * 获取所有列属性的枚举类,包括公用资源
     *
     * @return
     */
    public static List<DataResourceEnum> getColumnDataResource() {
        List<DataResourceEnum> list = new ArrayList<>();
        for (DataResourceEnum value : values()) {
            if (value.attrType.equals(DataAttrTypeEnum.DATA_ATTR_COLUMN) ||
                    value.attrType.equals(DataAttrTypeEnum.DATA_ATTR_ALL)) {
                list.add(value);
            }
        }
        return list;
    }

    /**
     * 获取所有表属性的枚举类,包括公用资源
     *
     * @return
     */
    public static List<DataResourceEnum> getTableDataResource() {
        List<DataResourceEnum> list = new ArrayList<>();
        for (DataResourceEnum value : values()) {
            if (value.attrType.equals(DataAttrTypeEnum.DATA_ATTR_TABLE) ||
                    value.attrType.equals(DataAttrTypeEnum.DATA_ATTR_ALL)) {
                list.add(value);
            }
        }
        return list;
    }

    /**
     * 根据code获取枚举类值
     *
     * @param code
     */
    public static DataResourceEnum getByCode(String code) {
        for (DataResourceEnum value : values()) {
            if (value.getCode().equals(code)) {
                return value;
            }
        }
        return null;
    }

}
