package com.dragoninfo.dcuc.app.service.sub.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.dragoninfo.dcuc.app.entity.sub.DataCla;
import com.dragoninfo.dcuc.app.enumresources.DataAuthTypeCodeEnum;
import com.dragoninfo.dcuc.app.dto.sub.DataLevelDTO;
import com.dragoninfo.dcuc.app.dto.sub.ExcelDataLevel;
import com.dragoninfo.dcuc.app.entity.sub.DataLevel;
import com.dragoninfo.dcuc.app.mapper.sub.DataLevelMapper;
import com.dragoninfo.dcuc.app.service.sub.IDataLevelService;
import com.dragoninfo.dcuc.app.util.ImportUtil;
import com.dragoninfo.dcuc.app.vo.DataLevelVo;
import com.dragoninfo.dcuc.duceap.facade.IDuceapUploadFacade;
import com.dragoninfo.dcuc.duceap.upload.dto.DocContentDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.collections.CollectionUtils;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.stereotype.Service;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 主客体授权-数据分级业务类
 * @author mazq
 * @date 2021/5/25
 */
@Service
public class DataLevelServiceImpl implements IDataLevelService {

    @Autowired
    private DataLevelMapper dataLevelMapper;

    @Override
    public ResponseStatus dataLevelAdd(DataLevelDTO dto) {
        String id = dto.getId();
        String levelCode = dto.getLevelCode();
        dto.setDataType(DataAuthTypeCodeEnum.AUTH_DATA_LEVEL.value);
        if(StringUtils.isBlank(levelCode)) {
            return ResponseStatus.fail("代码不能为空");
        }
        DataLevel exist = getByCode(levelCode);
        if(StringUtils.isBlank(id)) {
            if(null != exist) {
                return ResponseStatus.fail("代码已存在");
            }
            save(dto);
        } else {
            if(exist != null && !id.equals(exist.getId())) {
                return ResponseStatus.fail("代码已存在");
            }
            update(dto);
        }
        return ResponseStatus.success();
    }

    @Override
    public DataLevel update(DataLevelDTO dto) {
        DataLevel dataLevel = dataLevelMapper.selectById(dto.getId());
        if(null == dataLevel) {
            return null;
        }
        BeanUtils.copyProperties(dto, dataLevel,"createTime");
        dataLevel.setUpdateTime(new Date());
        dataLevelMapper.updateById(dataLevel);
        return dataLevel;
    }

    @Override
    public DataLevel save(DataLevelDTO dto) {
        DataLevel dataLevel = new DataLevel();
        BeanUtils.copyProperties(dto, dataLevel);
        dataLevel.setCreateTime(new Date());
        dataLevel.setDeleted(BooleanEnum.FALSE.value);
        dataLevelMapper.insert(dataLevel);
        return dataLevel;
    }

    @Override
    public DataLevel getById(String id) {
        return dataLevelMapper.selectById(id);
    }

    @Override
    public Boolean deleteById(String id) {
        int i = dataLevelMapper.deleteById(id);
        return i>0;
    }

    @Override
    public DataLevelDTO detailByCode(String levelCode) {
        LambdaQueryWrapper<DataLevel> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().eq(DataLevel::getLevelCode,levelCode);
        DataLevel dataLevel = dataLevelMapper.selectOne(queryWrapper);
        DataLevelDTO dto = new DataLevelDTO();
        if(null == dataLevel) {
            return dto;
        }
        BeanUtils.copyProperties(dataLevel, dto);
        return dto;
    }

    @Override
    public Page<DataLevelDTO> pageSearch(SearchDTO searchDTO) {
        Page<DataLevel> page = dataLevelMapper.pagingBySearchable(Searchable.toSearchable(searchDTO));
        List<DataLevelDTO> content = page.getContent().stream().map(item -> {
            DataLevelDTO dto = new DataLevelDTO();
            BeanUtils.copyProperties(item, dto);
            return dto;
        }).collect(Collectors.toList());
        return new PageImpl<>(content, page.getPageable(), page.getTotalElements());
    }


    @Override
    public void sync(List<DataLevelVo> list) {
        if(CollectionUtils.isEmpty(list)) {
            return;
        }
        //根据code去重
        Map<String, DataLevelVo> collect = Optional.ofNullable(list).orElse(new ArrayList<>())
                .stream()
                .collect(Collectors.toMap(DataLevelVo::getLevelCode, item -> item, (old, last) -> last));
        List<DataLevelVo> vos = new ArrayList<>(collect.values());
        for (DataLevelVo vo : vos) {
            DataLevel dataLevel = new DataLevel();
            BeanUtils.copyProperties(vo, dataLevel);
            dataLevel.setDataType(vo.getTypeCode());
            saveByCode(dataLevel);
        }

    }

    @Override
    public List<DataLevel> getByCodes(List<String> codes) {
        LambdaQueryWrapper<DataLevel> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().in(DataLevel::getLevelCode, codes);
        return dataLevelMapper.selectList(queryWrapper);
    }

    @Override
    public List<DataLevel> getAll() {
        return dataLevelMapper.selectList(null);
    }

    @Override
    public List<DataLevel> getByIds(List<String> ids) {
        if(CollectionUtils.isEmpty(ids)) {
            return new ArrayList<>();
        }
        LambdaQueryWrapper<DataLevel> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().in(DataLevel::getId, ids);
        return dataLevelMapper.selectList(queryWrapper);
    }


    private void saveByCode(DataLevel dataLevel) {
        DataLevel exist = this.getByCode(dataLevel.getLevelCode());
        if(null == exist) {
            dataLevel.setCreateTime(new Date());
            dataLevel.setDeleted(BooleanEnum.FALSE.value);
            dataLevelMapper.insert(dataLevel);
        }else {
            exist.setUpdateTime(new Date());
            exist.setLevelName(dataLevel.getLevelName());
            exist.setRemark(dataLevel.getRemark());
            dataLevelMapper.updateById(exist);
        }
    }

    private DataLevel getByCode(String levelCode) {
        LambdaQueryWrapper<DataLevel> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().eq(DataLevel::getLevelCode,levelCode);
        return dataLevelMapper.selectOne(queryWrapper);
    }

}
