package com.dragoninfo.dcuc.app.service.sub.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dragoninfo.dcuc.app.enumresources.DataAuthTypeCodeEnum;
import com.dragoninfo.dcuc.app.dto.sub.FieldSecDTO;
import com.dragoninfo.dcuc.app.entity.sub.FieldSec;
import com.dragoninfo.dcuc.app.mapper.sub.DataSecMapper;
import com.dragoninfo.dcuc.app.service.sub.IDataSecService;
import com.dragoninfo.dcuc.app.vo.DataLevelVo;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.commons.util.collections.CollectionUtils;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 主客体授权-数据安全等级业务类
 * @author mazq
 * @date 2021/5/25
 */
@Service
public class DataSecServiceImpl implements IDataSecService {

    @Autowired
    private DataSecMapper dataSecMapper;

    @Override
    public ResponseStatus dataLevelAdd(FieldSecDTO dto) {
        String id = dto.getId();
        String code = dto.getSecCode();
        dto.setDataType(DataAuthTypeCodeEnum.AUTH_DATA_SEC.value);
        if(StringUtils.isBlank(code)) {
            return ResponseStatus.fail("代码不能为空");
        }
        FieldSec exist = getByCode(code);
        if(StringUtils.isBlank(id)) {
            if(null != exist) {
                return ResponseStatus.fail("代码已存在");
            }
            save(dto);
        } else {
            if(null != exist && !id.equals(exist.getId())) {
                return ResponseStatus.fail("代码已存在");
            }
            update(dto);
        }

        return ResponseStatus.success();
    }

    @Override
    public FieldSec update(FieldSecDTO dto) {
        FieldSec dataSec = dataSecMapper.selectById(dto.getId());
        if (null == dataSec) {
            return null;
        }
        BeanUtils.copyProperties(dto, dataSec, "createTime");
        dataSec.setUpdateTime(new Date());
        dataSecMapper.updateById(dataSec);
        return dataSec;
    }

    @Override
    public FieldSec save(FieldSecDTO dto) {
        FieldSec dataSec = new FieldSec();
        BeanUtils.copyProperties(dto, dataSec);
        dataSec.setCreateTime(new Date());
        dataSec.setDeleted(BooleanEnum.FALSE.value);
        dataSecMapper.insert(dataSec);
        return dataSec;
    }

    @Override
    public FieldSec getById(String id) {
        return dataSecMapper.selectById(id);
    }

    @Override
    public Boolean deleteById(String id) {
        return dataSecMapper.deleteById(id) > 0;
    }

    @Override
    public FieldSecDTO detailByCode(String secCode) {
        LambdaQueryWrapper<FieldSec> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().eq(FieldSec::getSecCode, secCode);
        FieldSec dataSec = dataSecMapper.selectOne(queryWrapper);
        FieldSecDTO dto = new FieldSecDTO();
        if (null == dataSec) {
            return dto;
        }
        BeanUtils.copyProperties(dataSec, dto);
        return dto;
    }

    @Override
    public Page<FieldSecDTO> pageSearch(SearchDTO searchDTO) {
        Page<FieldSec> dataSecs = dataSecMapper.pagingBySearchable(Searchable.toSearchable(searchDTO));
        List<FieldSec> pageContent = dataSecs.getContent();
        List<FieldSecDTO> collect = pageContent.stream().map(item -> {
            FieldSecDTO dto = new FieldSecDTO();
            BeanUtils.copyProperties(item, dto);
            return dto;
        }).collect(Collectors.toList());
        Page<FieldSecDTO> page = new PageImpl<>(collect, dataSecs.getPageable(), dataSecs.getTotalElements());
        return page;
    }

    @Override
    public void sync(List<DataLevelVo> list) {
        if(CollectionUtils.isEmpty(list)) {
            return;
        }
        //根据code去重
        Map<String, DataLevelVo> collect = Optional.of(list).orElse(new ArrayList<>())
                .stream()
                .collect(Collectors.toMap(DataLevelVo::getLevelCode, item -> item, (old, last) -> last));
        List<DataLevelVo> vos = new ArrayList<>(collect.values());
        for (DataLevelVo vo : vos) {
            FieldSec dataSec = new FieldSec();
            dataSec.setSecCode(vo.getLevelCode());
            dataSec.setSecName(vo.getLevelName());
            dataSec.setDataType(vo.getTypeCode());
            saveByCode(dataSec);
        }

    }


    @Override
    public List<FieldSec> getByCodes(List<String> codes) {
        LambdaQueryWrapper<FieldSec> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().in(FieldSec::getSecCode, codes);
        return dataSecMapper.selectList(queryWrapper);
    }

    @Override
    public List<FieldSec> getAll() {
        return dataSecMapper.selectList(null);
    }

    @Override
    public List<FieldSec> getByIds(List<String> ids) {
        if(CollectionUtils.isEmpty(ids)) {
            return new ArrayList<>();
        }
        LambdaQueryWrapper<FieldSec> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().in(FieldSec::getId, ids);
        return dataSecMapper.selectList(queryWrapper);
    }

    private void saveByCode(FieldSec dataSec) {
        FieldSec exist = this.getByCode(dataSec.getSecCode());
        if(null == exist) {
            dataSec.setDeleted(BooleanEnum.FALSE.value);
            dataSec.setCreateTime(new Date());
            dataSecMapper.insert(dataSec);
        } else {
            exist.setUpdateTime(new Date());
            exist.setSecName(dataSec.getSecName());
            exist.setRemark(dataSec.getRemark());
            dataSecMapper.updateById(exist);
        }
    }

    private FieldSec getByCode(String secCode) {
        LambdaQueryWrapper<FieldSec> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select().eq(FieldSec::getSecCode, secCode);
        return dataSecMapper.selectOne(queryWrapper);
    }
}